/*****************************************************************************
* University of Southern Denmark
* Embedded C Programming (ECP)
*
* MODULENAME.: main.c
*
* PROJECT....: ECP
*
* DESCRIPTION: See module specification file (.h-file).
*
* Change Log:
******************************************************************************
* Date    Id    Change
* YYMMDD
* --------------------
* 0902012  KHA   Module created.
*
*****************************************************************************/


/***************************** Include files *******************************/
#include "inc\lm3s6965.h"
#include "inc\ecp_type.h"

/*****************************    Defines    *******************************/

#define PF1		1

/*****************************   Constants   *******************************/

/*****************************   Variables   *******************************/

/*****************************   Functions   *******************************/

void init_clk_system()
/*****************************************************************************
*   Input    : 	-
*   Output   : 	-
*   Function : 	
*****************************************************************************/
{
	INT32U dummy;
	// Init clock settings according to datasheet 
	// Set Clock speed to 50 Mhz.
	
	// Step 1
	// Set BYPASS bit
	SYSCTL_RCC_R |= SYSCTL_RCC_BYPASS;
	// Clear USESYS bits
	SYSCTL_RCC_R &= ~(SYSCTL_RCC_USESYSDIV);
	// Enable Main osc.
	SYSCTL_RCC_R |= SYSCTL_RCC_MOSCDIS;
	
	// Step 2
	// Clear PLL lock flag
	SYSCTL_RIS_R = ~(SYSCTL_RIS_PLLLRIS);
	// Set XTAL value, first clear bits then set 
	SYSCTL_RCC_R &= ~(SYSCTL_RCC_XTAL_M); 
	SYSCTL_RCC_R |= SYSCTL_RCC_XTAL_8MHZ;
	// Set OSCSRC, first clear bits then set 
	SYSCTL_RCC_R &= ~(SYSCTL_RCC_OSCSRC_M); 
	SYSCTL_RCC_R |= SYSCTL_RCC_OSCSRC_MAIN;
	// Clear PWRDN
	SYSCTL_RCC_R &= ~(SYSCTL_RCC_PWRDN);

	// delay
	for (dummy = 200000; dummy >0; dummy--);
		
	// Step 3
	// Set SYSDIV, first clear bits then set
	// Set to 50 Mhz.
	SYSCTL_RCC_R &= ~(SYSCTL_RCC_SYSDIV_M); 
	SYSCTL_RCC_R |= SYSCTL_RCC_SYSDIV_3;
	// Set USESYS bit
	SYSCTL_RCC_R |= SYSCTL_RCC_USESYSDIV;
	
	// Step 4
	// wait for PLL lock
	while(!(SYSCTL_RIS_R & SYSCTL_RIS_PLLLRIS));

	// Step 5
	// Clear BYPASS bit
	SYSCTL_RCC_R &= ~(SYSCTL_RCC_BYPASS);
	
	// delay
	for (dummy = 200000; dummy >0; dummy--);
		
}


void init_status_led(void)
/*****************************************************************************
*   Input    : 	-
*   Output   : 	-
*   Function : 	
*****************************************************************************/
{
  INT8S dummy;
  // Enable the GPIO port that is used for the on-board LED.
  SYSCTL_RCGC2_R |= SYSCTL_RCGC2_GPIOF;

  // Do a dummy read to insert a few cycles after enabling the peripheral.
  dummy = SYSCTL_RCGC2_R;

  // Set the direction as output.  
  GPIO_PORTF_DIR_R |= PF1;
  
  // Enable the GPIO pins for digital function.
  GPIO_PORTF_DEN_R |= PF1;
  
  // Set pin high
  GPIO_PORTF_DATA_R |= PF1;
}


void init_timer0(void)
/*****************************************************************************
*   Input    : 	-
*   Output   : 	-
*   Function : 	
*****************************************************************************/
{
	// Enable timer peripheral clock
	SYSCTL_RCGC1_R |= SYSCTL_RCGC1_TIMER0;
	// Disable timer
	TIMER0_CTL_R &= ~(TIMER_CTL_TAEN);
	// Enable 32-bit configuration
	TIMER0_CFG_R = 0x00000000;
	// Set to Periodic Timer mode, first clear then set
	TIMER0_TAMR_R &= ~(TIMER_TAMR_TAMR_M);
	TIMER0_TAMR_R |= TIMER_TAMR_TAMR_PERIOD;
	// Set Reload value. timeout = 1sek @ F_CPU = 50 Mhz.
	TIMER0_TAILR_R = 50000000;
	// Enable timer 0 int.
	TIMER0_IMR_R |= TIMER_IMR_TATOIM;
	
	// NVIC setup
	// program NVIC, vector number 35, Interrupt Number = 19
	// Clear pending interrupt
	NVIC_UNPEND0_R |= NVIC_UNPEND0_INT19;
	// Set Priority to 0x10, first clear then set. 
	NVIC_PRI4_R &= ~(NVIC_PRI4_INT19_M);
	NVIC_PRI4_R |= (NVIC_PRI4_INT19_M & (0x10<<NVIC_PRI4_INT19_S));
	// enable NVIC interrupt
	NVIC_EN0_R |= NVIC_EN0_INT19;
	
	// Enable and start timer
	TIMER0_CTL_R |= TIMER_CTL_TAEN;	
		
}
    	

void timer0_A_isr(void)
/*****************************************************************************
*   Input    : 	-
*   Output   : 	-
*   Function : 	
*****************************************************************************/
{
	// Clear Timer 0 int reguest
	TIMER0_ICR_R |= TIMER_ICR_TATOCINT;
	// Toggle status led
	GPIO_PORTF_DATA_R ^= PF1;
}


void enable_global_int()
{
	// enable interrupts.
	__asm("cpsie i");
}

void disable_global_int()
{
  // disable interrupts.
  __asm("cpsid i");
}


int main(void)
/*****************************************************************************
*   Input    : NONE
*   Output   : Returns 0
*   Function : Init hardware and then loop forever
******************************************************************************/
{	
  INT8U	dummy;
	
  disable_global_int();
	init_clk_system();
  init_status_led();
	init_timer0();
  enable_global_int();
	
    // Loop forever.
  while(1)
  {
  	dummy++; 	// For debug purpose
  }
  return( 0 );
}

/****************************** End Of Module *******************************/
